#ifndef __USB_CONFIG_H
#define	__USB_CONFIG_H

#define USB_EP0_BUFF_SIZE 8 // Either 8, 16, 32 or 64 bytes.

#define USB_MAX_NUM_INT 3
#define USB_MAX_EP_NUMBER 3

// Device Descriptor
#define USB_USER_DEVICE_DESCRIPTOR &device_dsc
#define USB_USER_DEVICE_DESCRIPTOR_INCLUDE extern ROM USB_DEVICE_DESCRIPTOR device_dsc

// Configuration Descriptors
#define USB_USER_CONFIG_DESCRIPTOR USB_CD_Ptr
#define USB_USER_CONFIG_DESCRIPTOR_INCLUDE extern ROM BYTE *ROM USB_CD_Ptr[]

// Ping Pong Mode
// Uncomment only one of the below options
//#define USB_PING_PONG_MODE USB_PING_PONG__NO_PING_PONG
#define USB_PING_PONG_MODE USB_PING_PONG__FULL_PING_PONG
//#define USB_PING_PONG_MODE USB_PING_PONG__EP0_OUT_ONLY
//#define USB_PING_PONG_MODE USB_PING_PONG__ALL_BUT_EP0

#define USB_FULL_PING_PONG
#define USB_EP0_OUT_ONLY

// USB Polling OR Interrupt mode
//#define USB_POLLING
#define USB_INTERRUPT

// USB Internal Pull Up
#define USB_PULLUP_OPTION USB_PULLUP_ENABLE
//#define USB_PULLUP_OPTION USB_PULLUP_DISABLED

// USB Tranceiver
#define USB_TRANSCEIVER_OPTION USB_INTERNAL_TRANSCEIVER
//#define USB_TRANSCEIVER_OPTION USB_EXTERNAL_TRANSCEIVER

// USB Speed
#define USB_SPEED_OPTION USB_FULL_SPEED
//#define USB_SPEED_OPTION USB_LOW_SPEED

// PID & VID
#define MY_VID 0x04D8
#define MY_PID 0x005E

// Status Stage Timeouts
#define USB_ENABLE_STATUS_STAGE_TIMEOUTS
// Approximate timeout in milliseconds, except when USB_POLLING mode is used, and USBDeviceTasks() is called at < 1kHz
// In this special case, the timeout becomes approximately:
// Timeout(ms) = ((1000 * (USB_STATUS_STAGE_TIMEOUT - 1)) / (USBDeviceTasks() polling frequency in Hz))
#define USB_STATUS_STAGE_TIMEOUT (BYTE)45

#define USB_SUPPORT_DEVICE

#define USB_NUM_STRING_DESCRIPTORS 3

// USB Interrupt Handlers
//#define USB_INTERRUPT_LEGACY_CALLBACKS
#define USB_ENABLE_ALL_HANDLERS
//#define USB_ENABLE_SUSPEND_HANDLER
//#define USB_ENABLE_WAKEUP_FROM_SUSPEND_HANDLER
//#define USB_ENABLE_SOF_HANDLER
//#define USB_ENABLE_ERROR_HANDLER
//#define USB_ENABLE_OTHER_REQUEST_HANDLER
//#define USB_ENABLE_SET_DESCRIPTOR_HANDLER
//#define USB_ENABLE_INIT_EP_HANDLER
//#define USB_ENABLE_EP0_DATA_HANDLER
//#define USB_ENABLE_TRANSFER_COMPLETE_HANDLER

// USB Device Class Usage
#define USB_USE_HID
#define USB_USE_CDC

// HID Endpoints
#define HID_INTF_ID 0x00
#define HID_EP 0x01 
#define HID_INT_OUT_EP_SIZE 0x40
#define HID_INT_IN_EP_SIZE 0x40
#define HID_NUM_OF_DSC 0x01
#define HID_RPT01_SIZE 0x1C // 28

// CDC Related
#define CDC_COMM_INTF_ID 0x01
#define CDC_COMM_EP 0x02
#define CDC_COMM_IN_EP_SIZE 0x0A

#define CDC_DATA_INTF_ID 0x02
#define CDC_DATA_EP 0x03
#define CDC_DATA_OUT_EP_SIZE 0x40
#define CDC_DATA_IN_EP_SIZE 0x40

#define USB_CDC_SET_LINE_CODING_HANDLER mySetLineCodingHandler
// #define USB_CDC_SUPPORT_HARDWARE_FLOW_CONTROL

// Send_Break command
// #define USB_CDC_SUPPORT_ABSTRACT_CONTROL_MANAGEMENT_CAPABILITIES_D2
// Set_Line_Coding, Set_Control_Line_State, Get_Line_Coding, and Serial_State commands
#define USB_CDC_SUPPORT_ABSTRACT_CONTROL_MANAGEMENT_CAPABILITIES_D1 

#endif
