#ifndef __USB_DESCRIPTORS_C
#define __USB_DESCRIPTORS_C

// Includes -----------------------------------------------------------------------------------------------------------

#include "USB/usb.h"
#include "USB/usb_function_hid.h"
#include "USB/usb_function_cdc.h"

// Device Descriptor --------------------------------------------------------------------------------------------------

ROM USB_DEVICE_DESCRIPTOR device_dsc = {
    sizeof(USB_DEVICE_DESCRIPTOR),  // Length of this descriptor
    USB_DESCRIPTOR_DEVICE,          // DEVICE descriptor type
    0x0200,                         // USB Spec Release Number (BCD Format)
    0xEF,                           // Class code (Miscellaneous)
    0x02,                           // Subclass code (Common Class)
    0x01,                           // Protocol code (Interface Association Descriptor)
    USB_EP0_BUFF_SIZE,              // Maximum packet size for endpoint 0
    MY_VID,                         // Vendor ID
    MY_PID,                         // Product ID
    0x0001,                         // Device release number (BCD Format)
    0x01,                           // Index of String Descriptor describing the manufacturer
    0x02,                           // Index of String Descriptor describing the product
    0x00,                           // Index of String Descriptor with the device's serial number
    0x01                            // Number of possible configurations
};

// Configuration Descriptor -------------------------------------------------------------------------------------------

ROM BYTE configDescriptor1[] = {
    // Configuration Descriptor
    0x09,                               // Size of this descriptor in bytes
    USB_DESCRIPTOR_CONFIGURATION,       // Configuration descriptor type
    DESC_CONFIG_WORD(0x6B),             // Total length of data for this configuration (0x006B = 107 bytes)
    0x03,                               // Number of interfaces in this configuration
    0x01,                               // Index value of this configuration
    0x01,                               // Configuration string index
    _DEFAULT | _SELF,                   // Attributes, see usb_device.h
    0x32,                               // Max power consumption (2X mA)

    // HID ------------------------------------------------------------------------------------------------------------

    // Interface Descriptor
    0x09,                               // Size of this descriptor in bytes
    USB_DESCRIPTOR_INTERFACE,           // Interface descriptor type
    0x00,                               // Interface number
    0x00,                               // Alternate setting number
    0x02,                               // Number of endpoints in this interface
    HID_INTF,                           // Class code
    0x00,                               // Subclass code
    HID_PROTOCOL_NONE,                  // Protocol code
    0x00,                               // Interface string index

    // HID Class-Specific Descriptor
    0x09,                               // Size of this descriptor in bytes (RRoj hack)
    DSC_HID,                            // HID descriptor type
    DESC_CONFIG_WORD(0x0111),           // HID specification release number in BCD format (1.11)
    0x00,                               // Country code
    0x01,                               // Number of class descriptors
    DSC_RPT,                            // Report descriptor type
    DESC_CONFIG_WORD(0x4A),             // Size of the report descriptor

    // Endpoint Descriptor
    0x07,
    USB_DESCRIPTOR_ENDPOINT,            // Endpoint descriptor
    HID_EP | _EP_IN,                    // Endpoint address
    _INTERRUPT,                         // Attributes
    DESC_CONFIG_WORD(0x40),             // Size
    0x01,                               // Interval

    // Endpoint Descriptor
    0x07,
    USB_DESCRIPTOR_ENDPOINT,            // Endpoint descriptor
    HID_EP | _EP_OUT,                   // Endpoint address
    _INTERRUPT,                         // Attributes
    DESC_CONFIG_WORD(0x40),             // Size
    0x01,                               // Interval

    // IAD ------------------------------------------------------------------------------------------------------------

    // Interface Association Descriptor
    0x08,                               // Size of this descriptor in bytes
    0x0B,                               // Interface association descriptor type
    0x01,                               // First associated interface
    0x02,                               // Number of contiguous associated interfaces
    COMM_INTF,                          // bInterfaceClass of the first interface
    ABSTRACT_CONTROL_MODEL,             // bInterfaceSubClass of the first interface
    V25TER,                             // bInterfaceProtocol of the first interface
    0x00,                               // Interface string index

    // CDC ------------------------------------------------------------------------------------------------------------

    // Interface Descriptor
    0x09,                               // Size of this descriptor in bytes
    USB_DESCRIPTOR_INTERFACE,           // Interface descriptor type
    CDC_COMM_INTF_ID,                   // Interface number
    0x00,                               // Alternate setting number
    0x01,                               // Number of endpoints in this interface
    COMM_INTF,                          // Class code
    ABSTRACT_CONTROL_MODEL,             // Subclass code
    V25TER,                             // Protocol code
    0x00,                               // Interface string index

    // CDC Class Specific
    sizeof(USB_CDC_HEADER_FN_DSC),      // Size of this descriptor in bytes (5)
    CS_INTERFACE,                       // bDescriptorType
    DSC_FN_HEADER,                      // bDescriptorSubtype
    0x20, 0x01,                         // bcdCDC

    // Abstract Control Management Functional Descriptor
    sizeof(USB_CDC_ACM_FN_DSC),         // Size of this descriptor in bytes (4)
    CS_INTERFACE,                       // bDescriptorType
    DSC_FN_ACM,                         // bDescriptorSubtype
    USB_CDC_ACM_FN_DSC_VAL,             // bmCapabilities: (see PSTN120.pdf Table 4)

    // Union Functional Descriptor
    sizeof(USB_CDC_UNION_FN_DSC),       // Size of this descriptor in bytes (5)
    CS_INTERFACE,                       // bDescriptorType
    DSC_FN_UNION,                       // bDescriptorSubtype
    0x01,                               // bControlInterface
    0x02,                               // bSubordinateInterface0

    // Call Management Functional Descriptor
    sizeof(USB_CDC_CALL_MGT_FN_DSC),    // Size of this descriptor in bytes (5)
    CS_INTERFACE,                       // bDescriptorType
    DSC_FN_CALL_MGT,                    // bDescriptorSubtype
    0x00,                               // bmCapabilities
    0x02,                               // bDataInterface

    // Endpoint Descriptor
    0x07,
    USB_DESCRIPTOR_ENDPOINT,            // Endpoint descriptor
    _EP02_IN,                           // Endpoint address
    _INTERRUPT,                         // Attributes
    CDC_COMM_IN_EP_SIZE, 0x00,          // Size
    0x02,                               // Interval

    // CDC Data Interface
    0x09,                               // Size of this descriptor in bytes
    USB_DESCRIPTOR_INTERFACE,           // Interface descriptor type
    0x02,                               // Interface number
    0x00,                               // Alternate setting number
    0x02,                               // Number of endpoints in this interface
    DATA_INTF,                          // Class code
    0x00,                               // Subclass code
    NO_PROTOCOL,                        // Protocol code
    0x00,                               // Interface string index

    // Endpoint Descriptor
    0x07,                               // Size of this descriptor in bytes
    USB_DESCRIPTOR_ENDPOINT,            // Endpoint descriptor
    _EP03_OUT,                          // Endpoint address
    _BULK,                              // Attributes
    DESC_CONFIG_WORD(0x40),             // Size
    0x00,                               // Interval

    // Endpoint Descriptor
    0x07,                               // Size of this descriptor in bytes
    USB_DESCRIPTOR_ENDPOINT,            // Endpoint descriptor
    _EP03_IN,                           // Endpoint address
    _BULK,                              // Attributes
    DESC_CONFIG_WORD(0x40),             // Size
    0x00                                // Interval
};

// Language code string descriptor
ROM struct {BYTE bLength; BYTE bDscType; WORD string[1];} sd000 = {
    sizeof(sd000),USB_DESCRIPTOR_STRING, {0x0409
}};

// Manufacturer string descriptor
ROM struct {BYTE bLength; BYTE bDscType; WORD string[9];} sd001 = {
sizeof(sd001),USB_DESCRIPTOR_STRING, {
    'S','p','a','r','k','B','u','z','z'
}};

// Product string descriptor
ROM struct {BYTE bLength;BYTE bDscType;WORD string[12];} sd002 = {
sizeof(sd002),USB_DESCRIPTOR_STRING, {
    'F','l','i','g','h','t',' ','S','t','i','c','k'
}};

// Array of configuration descriptors
ROM BYTE *ROM USB_CD_Ptr[] = {
    (ROM BYTE *ROM)&configDescriptor1
};

// Array of string descriptors
ROM BYTE *ROM USB_SD_Ptr[] = {
    (ROM BYTE *ROM)&sd000,
    (ROM BYTE *ROM)&sd001,
    (ROM BYTE *ROM)&sd002
};

// HID Descriptor
ROM struct {BYTE report[HID_RPT01_SIZE];} hid_rpt01 = {{
    0x05, 0x01,    // USAGE_PAGE (Generic Desktop)
    0x09, 0x04,    // USAGE (Joystick)
    0xa1, 0x01,    // COLLECTION (Application)
    0x09, 0x04,    //     USAGE (Joystick)
    0xa1, 0x00,    //     COLLECTION (Physical)
    0x05, 0x09,    //     USAGE_PAGE (Button)
    0x19, 0x01,    //     USAGE_MINIMUM (Button 1)
    0x29, 0x08,    //     USAGE_MAXIMUM (Button 8)
    0x15, 0x00,    //     LOGICAL_MINIMUM (0)
    0x25, 0x01,    //     LOGICAL_MAXIMUM (1)
    0x95, 0x08,    // REPORT_COUNT (8)
    0x75, 0x01,    // REPORT_SIZE (1)
    0x81, 0x02,    // INPUT (Data,Var,Abs)
    0xc0,          // END_COLLECTION
    0xc0           // END_COLLECTION
}};

#endif